/*---------------------------------------------------------------------------

    Written by the Personal Journal developers of Dow Jones & Company, Inc.

    Dow Jones makes no representations or warranties about 
    the suitability of this software, either express or 
    implied, including but not limited to the implied warranties 
    of merchantability, fitness for a particular purpose, 
    or non-infringement.  Dow Jones will not be liable for 
    any damages suffered by a user as a result of using, 
    modifying or distributing this software or its derivatives.


    NotebookTabBar.java

        A tab bar based on java.awt.Checkbox and java.util.CheckboxGroup


    Authors:

        rphall          Rick Hall
        jlee            James Lee


    Version Ident:

        $Header: /PjJavaClient/src/pj/awt/NotebookTabBar.java 7     3/22/96 9:44p Jlee $


    History:

        10-Dec-95   rphall      Initial Creation
         4-Mar-96   jlee        Major change: replaced Checkbox with the custom made Tab.
                                changed some functionality according to the replacement.
                                changed the class name form "CheckboxTabBar" to "TabNotebook"
                                changed the class name form "CheckboxTab" to "Notebooktab"
                                placed Notebooktab[CheckboxTab] in this file instead in a separate file.
        22-Mar-96   jlee        Used PjFinals's font variable for Tab's fonts and got rid of drawDottedRecTangle call.
        25-Mar-96   jlee        Modified layoutCOntainer so that it dosn't repeat the same layout change.
---------------------------------------------------------------------------*/

package pj.awt;

import pj.awt.Notebook;
import pj.awt.PjFinals;
import pj.awt.StringTabSpec;
import pj.awt.Tab;
import pj.awt.TabBar;
import pj.awt.TabSpec;

import collections.Assertable;
import collections.ImplementationCheckable;
import collections.ImplementationError;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.Panel;
import java.awt.Rectangle;
import java.lang.String;

/**
 * A tab bar implemented using NotebookTab.
 *
 * @see TabBar
 * @version 0.00 03-Jan-96
 * @author Rick Hall
*/
public class NotebookTabBar extends Panel implements Assertable, ImplementationCheckable, TabBar
    {

    // --- Instance variables

    /** The CheckboxGroup of a NotebookTabBar */
    Notebook        nbParent;
    NotebookTab     tabCurrent;

    // --- Public constructors

    /**
     * Construct a tab bar.
    */
    public NotebookTabBar(Notebook parent)
        {
        setLayout( new TabBarLayout() );

        assert(parent != null);
        nbParent = parent;

        System.out.println("Debug-NotebookTabBar:constructed");
        } // NotebookTabBar

    // --- Public operations

    /**
     * Append a tab and conditionally select it.
     *
     * @param tab The tab to append
     * @param selected True to make the new tab the selected tab,
     * false otherwise.
     * @exception ImplementationError if tab is not instanceof StringTabSpec.
    */
    public void appendTab(TabSpec tab, boolean selected) throws ImplementationError
        {
        assert(tab instanceof StringTabSpec);
        add(new NotebookTab((StringTabSpec)tab, selected) );
        }

    /**
     * Add a tab but do not select it.
     * @param tab The tab to append
     * @exception ImplementationError if tab is not instanceof StringTabSpec.
    */
    public void appendTab(TabSpec tab) throws ImplementationError
        {
        appendTab(tab,false);
        }

    /**
     * Enable a tab.
     * Subclasses must implement this method.
     *
     * @param name  The name of the tab to enable.
    */
    public void enableTab(String name)
        {
        NotebookTab nt = getTab(name);
        if (name != null) nt.enable();
        }

    /**
     * Enable all tabs.
    */
    public void enableTabs()
        {
        for (int i=0; i<countComponents(); i++)
            ((NotebookTab)getComponent(i)).enable();
        }


    /**
     * Disable a tab.
     * Subclasses must implement this method.
     *
     * @param name  The name of the tab to disable.
    */
    public void disableTab(String name)
        {
        NotebookTab cbt = getTab(name);
        if (name != null) cbt.disable();
        }

    /**
     * Disable all tabs.
    */
    public void disableTabs()
        {
        for (int i=0; i<countComponents(); i++)
            ((NotebookTab)getComponent(i)).disable();
        }

    /**
     * @return A specification for the currently selected tab.  If no
     * tab is selected, returns null.  If non-null, the returned
     * specification is an instance of StringTabSpec.
    */
    public TabSpec currentTabSpec()
        {
        StringTabSpec sts = null;
        for (int i=0; i<countComponents(); i++)
            {
            NotebookTab cbt = (NotebookTab)getComponent(i);
            if ( cbt.getState() )
                {
                sts = new StringTabSpec( cbt.strTabName, cbt.strPageName,
                        (String)cbt.getTabVisual() );

                break;
                } // if set

            } // for components

        return sts;
        } // currentTabSpec

    /**
     * Make a tab appear selected.
     *
     * @param name The name of the tab that should appear selected.
    */
    public void selectTab(String name)
        {
        NotebookTab cbt = getTab(name);
        if (cbt != null)
            {
            if ( tabCurrent != null )
                tabCurrent.setState(false);

            cbt.setState(true);
            tabCurrent = cbt;
            }
        }

    /**
     * Make a tab appear deselected.
     *
     * @param name The name of the tab that should appear deselected.
    */
    public void deselectTab(String name)
        {
        NotebookTab nt = getTab(name);
        if (name != null)
            nt.setState(false);
        }

    /**
     * Raise an exception if predicate is false.
     * @see collections.Assertable
    */
    public void assert(boolean predicate) throws ImplementationError
        {
        ImplementationError.assert(this, predicate);
        }

    /**
     * Checks that all components are instances of NotebookTab.
     * @exception ImplementationError if some component is not
     * an instance of NotebookTab
    */
    public void checkImplementation() throws ImplementationError
        {
        for (int i=0; i<countComponents(); i++)
            assert(getComponent(i) instanceof NotebookTab);
        }

    public boolean action(Event e, Object arg)
        {
        if (e.target instanceof NotebookTab)
            {
            String str = ((NotebookTab)e.target).getPageName();
            nbParent.page(str);
            }

        return true;
        }

    // --- Protected operations

    /**
     * Find a tab given its name.
     *
     * @param name The name of the tab.
     * @return null if no tab exists with the specified name.
     * @exception ImplementationError if some object other than
     * a NotebookTab is found with the specified name.
    */
    protected NotebookTab getTab(String name) throws ImplementationError
        {
        Component c = null;
        for (int i=0; i<countComponents(); i++)
            {
            c = getComponent(i);
            assert(c instanceof NotebookTab);

            String label = ((NotebookTab)c).getTabName();
            if (label == name)
                break;
            } // for

        return (NotebookTab)c;
        } // getTab



    } // TabBar

/*---------------------------------------------------------------------------

        Written by the Personal Journal developers of Dow Jones & Company, Inc.

    Dow Jones makes no representations or warranties about 
    the suitability of this software, either express or 
    implied, including but not limited to the implied warranties 
    of merchantability, fitness for a particular purpose, 
    or non-infringement.  Dow Jones will not be liable for 
    any damages suffered by a user as a result of using, 
    modifying or distributing this software or its derivatives.


    NotebookTab.java

        A tab implemented from a Checkbox.


        Authors:

        rphall          Rick Hall
        jlee            James Lee

        Version Ident:

        $Header: /PjJavaClient/src/pj/awt/NotebookTabBar.java 7     3/22/96 9:44p Jlee $


        History:

        10-dec-1995 rphalll     Initial Creation
         2-Mar-96   jlee        Replaced the old CheckboxTab with Notebooktab
                                changed functionality according to the replacement.
                                placed this class in this file from the separate file:CheckboxTab.java
---------------------------------------------------------------------------*/

/**
 * A tab implemented using Tab3D.
 *
 * @see pj.awt.Tab
 * @see java.awt.Checkbox
 * @version 0.00 03-Jan-96
 * @author Rick Hall
*/
class NotebookTab extends Tab3D implements Assertable, Tab
    {

     // --- Instance variables

    String label;
    String strTabName;
    String strPageName;

    // --- Public constructors

    /**
     * Construct a NotebookTab.
     * @param tab a string-based specification of this tab.
     * @param group the CheckboxGroup this Checkbox is in
     * @param state the initial state of this Checkbox
    */
    public NotebookTab( StringTabSpec tab, boolean selected)
        {
        super((String)tab.objVisSpec, selected);
        strTabName = tab.strTabName;
        strPageName = tab.strPageName;
        }

    // --- Public operations

    /**
     * Raise an exception if predicate is false.
     * @see collections.Assertable
    */
    public void assert(boolean predicate) throws ImplementationError
        {
        ImplementationError.assert(this, predicate);
        }

    /**
     * @param tabname The name for this tab.
    */
    public void setName(String tabname)
        {
        strTabName = tabname;
        }

    /**
     * @return The name for this tab.
    */
    public String getName()
        {
        return strTabName;
        }

    /**
     * Get a tab's name.
     * Equivalent to getName().
     *
     * @return The internal id of a tab within a TabBar.
     * @see getName
    */
    public String getTabName()
        {
        return getName();
        }

    /**
     * @param object The page name mapped to this tab.
     * @exception ImplementationError thrown if object is not an instance
     * of String
    */
    public void setMappedObject(Object object)
        throws ImplementationError
        {
        assert(object instanceof String);
        strPageName = (String)object;
        }

    /**
     * @return The page name mapped to this tab.
    */
    public Object getMappedObject()
        {
        return strPageName;
        }

    /**
     * Get the mapped name of a tab.
     * Equivalent to ((String)getMappedObject()).
     *
     * @return The name of the page which this tab references.
     * @see pj.util.Mappable#getMappedName
    */
    public String getPageName()
        {
        return (String)getMappedObject();
        }

    /**
     * @param visual The string presentation of this tab.
     * @exception ImplementationError if visual is not an instance of String
    */
    public void setTabVisual(Object visual)
        {
        assert(visual instanceof String);
        setLabel((String)visual);
        }

    /**
     * @return The String presentation of this tab.
    */
    public Object getTabVisual()
        {
        return getLabel();
        }


    /**
     * @param The String presentation of this tab.
    */
    public void setLabel( String lbl )
        {
        label = lbl;
        }

    /**
     * @return The String presentation of this tab.
    */
    public String getLabel()
        {
        return label;
        }



    } // Tab

/*---------------------------------------------------------------------------
    Written by the Personal Journal developers of Dow Jones & Company, Inc.

    Dow Jones makes no representations or warranties about 
    the suitability of this software, either express or 
    implied, including but not limited to the implied warranties 
    of merchantability, fitness for a particular purpose, 
    or non-infringement.  Dow Jones will not be liable for 
    any damages suffered by a user as a result of using, 
    modifying or distributing this software or its derivatives.


    @(#)Tab3D.java   0.00 2-Mar-96

        A Tab3D that implements Tabs in NotebookTab.

    Authors:

        jlee        James Lee

    Version Ident:

        $Header: /PjJavaClient/src/pj/awt/NotebookTabBar.java 7     3/22/96 9:44p Jlee $

    History:

         2-Mar-96    jlee   Initial creation.

---------------------------------------------------------------------------*/

/**
 * Tab3D implements "bottom" Tabs
 *
 * @version     0.00, 3/2/96
 * @author      James Lee
 */
class Tab3D extends Canvas {

    // --- Class variables
    public static  final boolean    UP = true;
    public static  final boolean    DOWN = false;
    public static  final int        FIRSTTAB = 1;
    public static  final int        LASTTAB = 2;
    public static  final int        MIDDLETAB = 3;

    // --- Instance variables
    private boolean  bUp;
    private String   strTabName;

    private final int nTabHeight = PjFinals.nTab3DHeight; //fixed
    private int       nTabWidth;                         //dynamic

    private int tabPosition;


    // --- Public constructors
    /**
     * Constructs a new Tab3D with a tab name.
     * @param strName A name for this tab
     */
    public Tab3D( String strName )
        {
        init( strName );
        }

    /**
     * Constructs a new Tab3D with a tab name and state(Up or Down).
     * @param strName A name for this tab
     * @param bState  A state that indicates wether this tab is up or down.
     */
    public Tab3D( String strName, boolean bState )
        {
        init( strName );
        bUp = bState;
        }


    // --- Public operations




    /**
     * Returns the current state of this tab:Up or Down.
     */
    public boolean getState()
        {
        return bUp;
        }

    /**
     * Sets the current state of this tab.
     */
    public void setState( boolean state)
        {
        if ( bUp != state )
            {
            bUp = state;
            repaint();
            }
        }

    /**
     * sets this tab's position, only two choices:First or Last.
     * Default value for hgap and vgap is 0.
     */
    public void setPosition( int pos )
        {
        tabPosition = pos;
        }

    /**
     * Inform all active Parent containers of a mouse press.
     *
     * @param evt An Event object
     * @param x X coordinate of mouse button press
     * @param y Y coordinate of mouse button press
     */
    public boolean mouseDown(Event evt, int x, int y)
        {
        postEvent( new Event( this, Event.ACTION_EVENT, strTabName ) );

        return true;
        }

    /**
     * Paint the canvas with the tab drawing.
    */
    public void paint(Graphics g)
        {
        int i;
        int off;
        int nOffset = 5; //right bottom edge slope degree
        int nNameWidth;
        int nNameHeight;

        if ( bUp )//selected
            {
            //Top edge
            g.setColor( Color.lightGray );
            g.drawLine( 0, 0, nTabWidth, 0 );
            g.drawLine( 0, 1, nTabWidth, 1 );
            g.drawLine( 0, 2, nTabWidth, 2 );
            g.drawLine( 0, 3, nTabWidth, 3 );

            //Right edge 1
            g.setColor( Color.black );
            g.drawLine( nTabWidth, 2, nTabWidth, nTabHeight - nOffset );

            //Right edge 2
            g.setColor( Color.darkGray );
            g.drawLine( nTabWidth - 1, 2, nTabWidth - 1, nTabHeight - nOffset - 1 );

            //Right edge 3
            g.setColor( Color.gray );
            g.drawLine( nTabWidth - 2, 2, nTabWidth - 2, nTabHeight - nOffset - 1 );

            //Right edge 4
            g.setColor( Color.gray );
            g.drawLine( nTabWidth - 3, 2, nTabWidth - 3, nTabHeight - nOffset - 1 );

            //Bottom right edge 1
            g.setColor( Color.black );
            g.drawLine( nTabWidth, nTabHeight - nOffset, nTabWidth - nOffset, nTabHeight );
            //Bottom right edge 2
            g.setColor( Color.darkGray );
            g.drawLine( nTabWidth - 1, nTabHeight - nOffset, nTabWidth - nOffset - 1, nTabHeight );
            //Bottom right edge 3
            g.setColor( Color.darkGray );
            g.drawLine( nTabWidth - 2, nTabHeight - nOffset, nTabWidth - nOffset - 2, nTabHeight );
            //Bottom right edge 4
            g.setColor( Color.gray );
            g.drawLine( nTabWidth - 3, nTabHeight - nOffset, nTabWidth - nOffset - 3, nTabHeight );
            //Bottom right edge 5
            g.setColor( Color.gray );
            g.drawLine( nTabWidth - 3, nTabHeight - nOffset - 1, nTabWidth - nOffset - 3, nTabHeight - 1 );
            //Bottom right edge 6
            g.setColor( Color.gray );
            g.drawLine( nTabWidth - 3, nTabHeight - nOffset - 2, nTabWidth - nOffset - 3, nTabHeight - 2 );

            //Bottom edge 1
            g.setColor( Color.black );
            g.drawLine( nTabWidth - nOffset, nTabHeight, nOffset, nTabHeight );
            //Bottom edge 2
            g.setColor( Color.darkGray );
            g.drawLine( nTabWidth - nOffset - 1, nTabHeight - 1, nOffset + 1, nTabHeight - 1 );
            //Bottom edge 3
            g.setColor( Color.gray );
            g.drawLine( nTabWidth - nOffset - 2, nTabHeight - 2, nOffset + 2, nTabHeight - 2 );
            //Bottom edge 4
            g.setColor( Color.gray );
            g.drawLine( nTabWidth - nOffset - 3, nTabHeight - 3, nOffset + 3, nTabHeight - 3 );

            //Bottom left edge 1
            g.setColor( Color.darkGray );
            g.drawLine( nOffset, nTabHeight, 0, nTabHeight - nOffset );

            //Bottom left edge 2
            g.setColor( Color.gray );
            g.drawLine( nOffset, nTabHeight - 1, 0, nTabHeight - nOffset - 1 );

            //Bottom left edge 3
            g.setColor( Color.white );
            g.drawLine( nOffset, nTabHeight - 2, 2, nTabHeight - nOffset );


            g.setColor( Color.white );
            off = (FIRSTTAB == tabPosition) ? 1: 4;

            //Left edge 1
            g.drawLine( 0, nTabHeight - nOffset, 0, off );
            //Left edge 2
            g.drawLine( 1, nTabHeight - nOffset, 1, off );

            //Draw tab name
            g.setColor( Color.black );
            g.setFont( PjFinals.fntUnfColumnBold );

            FontMetrics fmTab = g.getFontMetrics();
            nNameWidth = fmTab.stringWidth( strTabName );
            nNameHeight = fmTab.getHeight();

            g.drawString( strTabName, (nTabWidth - nNameWidth) / 2 - 1,
                          nNameHeight + (nTabHeight - nNameHeight - 5) / 2  );
            /*
            drawDottedRecTangle( (nTabWidth - nNameWidth) / 2 - 5, //X1
                                  nNameHeight + (nTabHeight - nNameHeight - 5) / 2 + 4, //Y1
                                  (nTabWidth - nNameWidth) / 2 + nNameWidth + 3, //X2
                                  nNameHeight / 3 + (nTabHeight - nNameHeight - 5) / 2 - 5 //Y2
                                );
            */
            }
        else //unselected
            {
            //Top edge 1
            g.setColor( Color.lightGray );
            g.drawLine( 0, 0, nTabWidth, 0 );

            //Top edge 2
            g.setColor( Color.gray );
            g.drawLine( 0, 1, nTabWidth, 1 );

            //Top edge 3
            g.setColor( Color.gray );
            g.drawLine( 0, 2, nTabWidth, 2 );

            //Top edge 4
            g.setColor( Color.darkGray );
            g.drawLine( 0, 3, nTabWidth, 3 );

            g.setColor( Color.white );
            off = 2;

            //Bottom right edge 1
            g.setColor( Color.black );
            g.drawLine( nTabWidth, nTabHeight - nOffset, nTabWidth - nOffset, nTabHeight );

            //Bottom right edge 2
            g.setColor( Color.gray );
            g.drawLine( nTabWidth - 1, nTabHeight - nOffset, nTabWidth - nOffset - 1, nTabHeight );

            //Bottom right edge 3
            g.setColor( Color.gray );
            g.drawLine( nTabWidth - 1, nTabHeight - nOffset - 1, nTabWidth - nOffset - 1, nTabHeight - 1 );

            //Right edge 1
            g.setColor( Color.black );
            g.drawLine( nTabWidth, off, nTabWidth, nTabHeight - nOffset );

            //Right edge 2
            g.setColor( Color.black );
            g.drawLine( nTabWidth - 1, off, nTabWidth - 1, nTabHeight - nOffset );

            //Right edge 3
            g.setColor( Color.gray );
            g.drawLine( nTabWidth - 2, off, nTabWidth - 2, nTabHeight - nOffset );

            //Bottom edge 1
            g.setColor( Color.black );
            g.drawLine( nTabWidth - nOffset, nTabHeight, nOffset, nTabHeight );

            //Bottom edge 2
            g.setColor( Color.black );
            g.drawLine( nTabWidth - nOffset - 1, nTabHeight - 1, nOffset + 1, nTabHeight - 1 );

            //Bottom edge 3
            g.setColor( Color.gray );
            g.drawLine( nTabWidth - nOffset - 1, nTabHeight - 2, nOffset + 1, nTabHeight - 2 );

            //Left edge 1
            g.setColor( Color.white );
            g.drawLine( 0, nTabHeight - nOffset, 0, 6 );

            //Left edge 2
            //g.setColor( Color.white );
            //g.drawLine( 1, nTabHeight - nOffset, 1, 4 );

            //Bottom left edge 1
            g.setColor( Color.black );
            g.drawLine( nOffset, nTabHeight, 0, nTabHeight - nOffset );

            //Bottom left edge 2
            g.setColor( Color.darkGray );
            g.drawLine( nOffset + 1, nTabHeight, 1, nTabHeight - nOffset );

            //Draw tab name
            g.setColor( Color.black );
            g.setFont( PjFinals.fntUnfColumnPlain );

            FontMetrics fmTab = g.getFontMetrics();
            nNameWidth = fmTab.stringWidth( strTabName );
            nNameHeight = fmTab.getHeight();

            g.drawString( strTabName, (nTabWidth - nNameWidth) / 2 ,
                          nNameHeight + (nTabHeight - nNameHeight - 4) / 2  );
            }
        }

    /**
     * Reshapes the Component to the specified bounding box.
     * @param x the x coordinate
     * @param y the y coordinate
     * @param width the width of the component
     * @param height the height of the component
     * @see #bounds
     * @see #move
     * @see #resize
     */
    public synchronized void reshape(int x, int y, int width, int height)
        {
        nTabWidth = width;
        super.reshape(x, y, nTabWidth, nTabHeight);
        repaint();
        }

    // --- Private operations
    private void init( String strName )
        {
        bUp = DOWN;
        strTabName = strName;
        tabPosition = MIDDLETAB;
        }
    }//Tab3D



/*---------------------------------------------------------------------------

    Written by the Personal Journal developers of Dow Jones & Company, Inc.

    Dow Jones makes no representations or warranties about 
    the suitability of this software, either express or 
    implied, including but not limited to the implied warranties 
    of merchantability, fitness for a particular purpose, 
    or non-infringement.  Dow Jones will not be liable for 
    any damages suffered by a user as a result of using, 
    modifying or distributing this software or its derivatives.


    @(#)TabBarLayout.java   0.00 5-Feb-96

        A TabBarLayout that manages Tabs layout.

    Authors:

        jlee        James Lee

    Version Ident:

        $Header: /PjJavaClient/src/pj/awt/NotebookTabBar.java 7     3/22/96 9:44p Jlee $

    History:

         2-Mar-96    jlee   Initial creation.

---------------------------------------------------------------------------*/

/**
 * TabBarLayout is used to layout Tabs in a panel.
 * @version     0.00, 3/2/96
 * @author      James Lee
 */
class TabBarLayout implements LayoutManager {

    // --- Instance variables
    private int hgap;
    private int vgap;

    private int nWidth;
    private int nHeight;

    private Rectangle recLast = new Rectangle(0, 0, 0, 0);

    // --- Public constructors

    /**
     * Constructs a new TabBarLayout with a centered alignment.
     * Default value for hgap and vgap is 0.
     */
    public TabBarLayout()
        {
        this(0, 0);
        }

    /**
     * Constructs a new TabBarLayout with the specified alignment and gap
     * values.
     * @param hgap the horizontal gap variable
     * @param vgap the vertical gap variable
     */
    public TabBarLayout(int hgap, int vgap)
        {
        this.hgap = hgap;
        this.vgap = vgap;
        }


    // --- Public operations

    /**
     * Adds the specified component to the layout. Not used by this class.
     * @param name the name of the component
     * @param comp the the component to be added
     */
    public void addLayoutComponent(String name, Component comp)
        {
        }

    /**
     * Removes the specified component from the layout. Not used by
     * this class.
     * @param comp the component to remove
     */
    public void removeLayoutComponent(Component comp)
        {
        }

    /**
     * Returns the preferred dimensions for this layout given the components
     * in the specified target container.
     * @param target the component which needs to be laid out
     * @see Container
     * @see #minimumLayoutSize
     */
    public Dimension preferredLayoutSize(Container target)
        {
        Dimension dim = new Dimension(0, 0);
        int nmembers = target.countComponents();

        for (int i = 0 ; i < nmembers ; i++) {
            Component m = target.getComponent(i);
            if (m.isVisible()) {
                Dimension d = m.preferredSize();
                dim.height = Math.max(dim.height, d.height);

                if (i > 0) {
                    dim.width += hgap;
                }

                dim.width += d.width;
            }
        }

        Insets insets = target.insets();
        dim.width += insets.left + insets.right + hgap*2;
        dim.height += insets.top + insets.bottom + vgap*2;
        return dim;
        }

    /**
     * Returns the minimum dimensions needed to layout the components
     * contained in the specified target container.
     * @param target the component which needs to be laid out
     * @see #preferredLayoutSize
     */
    public Dimension minimumLayoutSize(Container target)
        {
        Dimension dim = new Dimension(0, 0);
        int nmembers = target.countComponents();

        for (int i = 0 ; i < nmembers ; i++) {
            Component m = target.getComponent(i);

            if (m.isVisible()) {
                Dimension d = m.minimumSize();
                dim.height = Math.max(dim.height, d.height);

                if (i > 0) {
                    dim.width += hgap;
                }
                dim.width += d.width;
            }
        }
        Insets insets = target.insets();
        dim.width += insets.left + insets.right + hgap*2;
        dim.height += insets.top + insets.bottom + vgap*2;

        return dim;
        }


    /**
     * Lays out the container. This method will actually reshape the
     * components in the target in order to satisfy the constraints of
     * the TabBarLayout object.
     * @param target the specified component being laid out.
     * @see Container
     */
    public void layoutContainer(Container target)
        {
        Component m;
        int       i;

        Insets insets = target.insets();
        Rectangle rec = target.bounds();

        if ( target.isValid() &&
             rec.x == recLast.x && rec.y == recLast.y &&
             rec.width == recLast.width && rec.height == recLast.height  )
            return;
        else
            recLast = rec;

        nWidth = rec.width;
        nHeight = rec.height;

        int nmembers = target.countComponents();
        int tabWidth = rec.width / nmembers - 1;

        if ( nmembers > 0 )
            {
            ((Tab3D)target.getComponent(0)).setPosition( Tab3D.FIRSTTAB );

            for (i = 0 ; i < nmembers - 1 ; i++)
                {
                m = target.getComponent(i);
                if ( m.isVisible() )
                    {
                    m.reshape( i * (tabWidth + 1), 0, tabWidth, 20 );
                    }
                }

            m = target.getComponent(i);

            if ( m.isVisible() )
                {
                m.reshape( i * (tabWidth + 1), 0, rec.width - i * (tabWidth + 1), 0 );
                ((Tab3D)m).setPosition( Tab3D.LASTTAB );
                }
            }
        }

    /**
     * Returns the String representation of this TabBarLayout's values.
     */
    public String toString()
        {
        return  getClass().getName() + "[hgap=" + hgap + ",vgap=" + vgap + "]" +
                "[width=" + nWidth + ",height=" + nHeight + "]";
        }

    }//TabBarLayout

