/*---------------------------------------------------------------------------

    Written by the Personal Journal developers of Dow Jones & Company, Inc.

    Dow Jones makes no representations or warranties about 
    the suitability of this software, either express or 
    implied, including but not limited to the implied warranties 
    of merchantability, fitness for a particular purpose, 
    or non-infringement.  Dow Jones will not be liable for 
    any damages suffered by a user as a result of using, 
    modifying or distributing this software or its derivatives.



    @(#)WordWrapper.java  0.00 29-Dec-95

        A utility class that wraps unbroken text into lines.


    Authors:

        rphall   Rick Hall


    Version Ident:

        $Header$


    History:

        0.00 29-Dec-95  rphall   Initial Creation

---------------------------------------------------------------------------*/

package pj.awt;

import collections.Assertable;
import collections.ImplementationError;
import java.awt.FontMetrics;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 *  A utility that wraps unbroken text into lines.
 *
 *  @version    0.00 29-Dec-95
 *  @author     rphall
*/
public class WordWrapper implements Assertable
    {

    // --- Class variables

    /**
     *  Default set of line-delimiting characters
    */
    public static final char[] NEWLINE = {'\r','\n'};

    /**
     *  Default set of word-delimiting characters
    */
    public static final char[] WORDBRK = {'\t',' ','-'};
    
        
    private static final String  CR             = "\r";
    private static final String  LF             = "\n";
    private static final String  CRLF           = CR + LF;
    private static final boolean bReturnTokens  = true; 

    // --- Instance variables
   
    private FontMetrics fontMetrics;
    private String[]    astrLineDelim;
    private String[]    astrWordDelim;
    private String      strDelim;

    // --- Public constructors

    /**
     *  Constructs a WordWrapper with default line and word delimiters
     *
     *  @param fm   The metrics of the font used for text display
    */
    public WordWrapper(FontMetrics fm)
        {
        this(fm,NEWLINE,WORDBRK); 
        }

    /**
     *  Constructs a WordWrapper with default line and word delimiters
     *
     *  @param fm           The metrics of the font used for text display
     *  @param lineDelim    An array of custom line delimiters
     *  @param wordDelim    An array of custom word delimiters
    */
    public WordWrapper(FontMetrics fm, char[] lineDelim, char[] wordDelim)
        {
        fontMetrics = fm;
        astrLineDelim = new String[lineDelim.length];

        for (int i=0; i<lineDelim.length; i++)
            astrLineDelim[i] = new String(lineDelim,i,1);
        astrWordDelim = new String[wordDelim.length];

        for (int i=0; i<wordDelim.length; i++)
            astrWordDelim[i] = new String(wordDelim,i,1);
        strDelim = new String(lineDelim) + new String(wordDelim);
        }

    // --- Public operations

    /*
     *  Throws an ImplementationError if predicate is false.
     *
     *  @param predicate    Boolean expression to be checked.
     *  @see collections.ImplementationError
     *  @see collections.Assertable
    */
    public void assert(boolean predicate)
        throws ImplementationError
        {
        ImplementationError.assert(this, predicate);
        }

    /**
     *  Formats unbroken text into lines.  Returns the formatted text
     *  as a single string with embedded '\n' characters.
     *
     *  @param strRawText   The input text, length > 0
     *  @param linewidth    The desired line width
    */
    public String formatToString(String strRawText, int linewidth)
        {

        StringTokenizer st = new StringTokenizer(
                strRawText, strDelim, bReturnTokens);

        String strFmtText = "";
        String strLine = "";
        String strToken = null;
        int currentWidth = 0;
        int checkWidth   = 0;

        TokenLoop:
        for (int i=0; ; i++)
            {
            // Implementation note: countTokens() does not seem to be
            // working correctly.  Hence this loop continues until an
            // exception, rather than using countTokens to limit iterations.
            try {
                strToken = st.nextToken();
                }
            catch (NoSuchElementException e)
                {
                // throw new ImplementationError();
                break TokenLoop;
                }

            // Line delimiters reset a line length calc
            for (int j=0; j<astrLineDelim.length; j++)
                if ( strToken.equals(astrLineDelim[j]) )
                    {
                    strFmtText += strToken;
                    strLine = "";
                    currentWidth = 0;
                    continue TokenLoop;
                    }

            // Check linewidth with the addition of the new token
            checkWidth = currentWidth +
                    fontMetrics.stringWidth(strToken);

            // If the checked width is less than a line width,
            // add the new token to the current line and continue.
            if ( checkWidth < linewidth )
                {
                strFmtText += strToken;
                strLine += strToken;
                currentWidth = checkWidth;
                continue TokenLoop;
                }
            
            // If the checked width is longer than a line width,
            // check if the new token is a word delimiter.  Extra
            // word delimiters are OK at the end of a line.
            for (int j=0; j<astrWordDelim.length; j++)
                if ( strToken.equals(astrWordDelim[j]) )
                    {
                    strFmtText += strToken;
                    strLine += strToken;
                    currentWidth = checkWidth;
                    continue TokenLoop;
                    }

            // If the checked width is longer than a line width, avoid
            // pathologies by making sure the current line has at least
            // one token anyway.
            if ( currentWidth == 0 )
                {
                strFmtText += strToken;
                strLine += strToken;
                currentWidth = checkWidth;
                continue TokenLoop;
                }

            // If the checked width is longer than a line width,
            // and the current line has at least one token, append
            // a line break and the new token to the output.  Reset
            // the line length calculation.
            strFmtText += "\n" + strToken;
            strLine = strToken;
            currentWidth = fontMetrics.stringWidth(strToken);
            continue TokenLoop;

            } // for TokenLoop

        return strFmtText;

        } // formatToString

    /**
     *  Formats unbroken text into lines.  Returns the formatted text
     *  as an array of strings, each string representing a single line.
     *  Line delimiters are stripped from the text.  The sequence "\r\n"
     *  is treated as a single line delimiter.  Throws an implementation
     *  error if input text is empty.
     *
     *  @param strRawText   The input text, length > 0
     *  @param linewidth    The desired line width
     *  @see collections.ImplementationError
    */
    public String[] formatToArray(String strRawText, int linewidth)
        throws ImplementationError
        {
        // Precondition
        assert( strRawText.length() != 0 );

        // Replace CRLF with LF
        String text    = strRawText;
        int nextCRLF   = text.indexOf(CRLF);
        int nextSubstr = 0;

        while ( nextCRLF != -1 )
            {
            nextSubstr = text.length() - CRLF.length();
            if ( nextSubstr == 0 )
                {
                text = text.substring(0,nextCRLF) + LF;
                assert( text.indexOf(CRLF) == -1 );
                }
            else
                {
                text = text.substring(0,nextCRLF) + LF
                        + text.substring(nextSubstr);
                }

            nextCRLF  = text.indexOf(CRLF);

            } // while CRLF

        // Format into lines
        String[] strings;
        if ( text.length() == 0 )
            {
            strings = new String[1];
            strings[0] = "";
            }
        else
            {
            Vector vStrings = new Vector();
            String[] astrFmt = formatLine(text,linewidth);
            vStrings.addElement(astrFmt[0]);
            while ( astrFmt[1].length() > 0 )
                {
                astrFmt = formatLine(astrFmt[1],linewidth);
                vStrings.addElement(astrFmt[0]);
                }
            strings = new String[vStrings.size()];
            for ( int i=0; i<strings.length; i++ )
                strings[i] = (String)vStrings.elementAt(i);

            } // formatToArray

        return strings;

        } // formatToStringArray


    // --- Private operations

    /*
     *  Returns a line and prunes the line from the input text.
     *  Line delimiters are stripped from the line.  This operation will
     *  treat the sequence "\r\n" as two line delimiters, rather than one.
     *  If this is undesirable, replace all CRLF sequences by single LF
     *  sequences before invoking this operation.
     *
     *  @return String[0]   An extracted line
     *  @return String[1]   The input text pruned of the extracted line.
    */
    private String[] formatLine(
            String text,        // In:  text from which to extract a line
                                // Out: text minus the extracted line
            int desiredWidth)   // In:  the desired width of a line
        throws ImplementationError
        {

        // Check for pathology
        assert( text.equals("") == false );
        assert( desiredWidth > 0 );

        StringTokenizer st = new StringTokenizer(
                text, strDelim, bReturnTokens);

        String line       = "";   // The extracted line
        String strToken   = null; // A new token from text
        int currentWidth  = 0;    // The current width of the extracted line
        int checkWidth    = 0;    // The currentWidth plus a new token width
        int numLineChars  = 0;    // The number of chars in current line

        TokenLoop:
        for (int i=0; ; i++)
            {
            // Implementation note: countTokens() does not seem to be
            // working correctly.  Hence this loop continues until an
            // exception, rather than using countTokens to limit iterations.
            try {
                strToken = st.nextToken();
                }
            catch (NoSuchElementException e)
                {
                break TokenLoop;
                }

            // Line delimiters reset a line length calc
            for (int j=0; j<astrLineDelim.length; j++)
                if ( strToken.equals(astrLineDelim[j]) )
                    {
                    numLineChars += strToken.length();
                    break TokenLoop;
                    }

            // Check desiredWidth with the addition of the new token
            checkWidth = currentWidth + fontMetrics.stringWidth(strToken);

            // If the checked width is less than a line width,
            // add the new token to the current line and continue.
            if ( checkWidth < desiredWidth )
                {
                line += strToken;
                currentWidth = checkWidth;
                numLineChars += strToken.length();
                continue TokenLoop;
                }
            
            // If the checked width is longer than a line width,
            // check if the new token is a word delimiter.  Extra
            // word delimiters are OK at the end of a line.
            for (int j=0; j<astrWordDelim.length; j++)
                if ( strToken.equals(astrWordDelim[j]) )
                    {
                    line += strToken;
                    currentWidth = checkWidth;
                    numLineChars += strToken.length();
                    continue TokenLoop;
                    }

            // Avoid pathology by making sure the current line has
            // at least one token.
            if ( currentWidth == 0 )
                {
                line += strToken;
                currentWidth = checkWidth;
                numLineChars += strToken.length();
                continue TokenLoop;
                }

            // If the checked width is longer than a line width,
            // and the current line has at least one token, the line
            // is finished. The new token starts the returned text.
            break TokenLoop;

            } // for TokenLoop

        assert( numLineChars > 0 );
        assert( numLineChars <= text.length() );
        assert( line.length() <= text.length() );

        if ( numLineChars == text.length() )
            text = "";
        else
            text = text.substring(numLineChars);

        String[] strs = new String[2];
        strs[0] = line;
        strs[1] = text;

        return strs;

        } // formatLine

   

    } // WordWrapper
